﻿using Microsoft.Data.SqlClient;
using SqlDataReaderADONET.Models;
using SqlDataReaderADONET.PashaCode;
using System.Collections.Generic;
using System.Data;

namespace SqlDataReaderADONET.DirectSql
{
    public class SqlQueries
    {
        // Универсальные запросы к базе данных
        public static DataTable CreateQuerySqlOneTable(string queryString, List<SqlParameter> parameters = null)
        {
            DataTable result = CreateQuerySqlSeveralTables(queryString, parameters)[0];

            return result;
        }


        // Универсальные запросы к базе данных
        public static List<DataTable> CreateQuerySqlSeveralTables(string queryString, List<SqlParameter> parameters = null)
        {
            List<DataTable> listTables = new(); // C# 9.0

            string connectionString = Constants.SqlConnectionIntegratedSecurity;

            using SqlConnection connection = new(connectionString); // C# 9.0
            using SqlCommand command = new(queryString, connection); // C# 9.0

            if (parameters != null)
            {
                foreach (SqlParameter p in parameters)
                {
                    command.Parameters.Add(p);
                }
            }

            connection.Open();


            using SqlDataReader reader = command.ExecuteReader(CommandBehavior.KeyInfo /* без этого имя таблиц не передаются*/);

            while (reader.IsClosed == false)
            {
                // Если имя таблицы не указывать, название определяется из метаданных
                DataTable result = new(); // C# 9.0
                result.Load(reader);

                listTables.Add(result);
            }


            // Очистка кеша от предыдущей коллекции комманд.
            command.Parameters.Clear();

            return listTables;
        }


        // Извлечение данных SqlDataReader в свойства класса-модели
        public static List<Table> CreateQuerySqlDataReaderModel(string queryString, List<SqlParameter> parameters = null)
        {
            string connectionString = Constants.SqlConnectionIntegratedSecurity;

            using SqlConnection connection = new(connectionString); // C# 9.0
            using SqlCommand command = new(queryString, connection); // C# 9.0

            if (parameters != null)
            {
                foreach (SqlParameter p in parameters)
                {
                    command.Parameters.Add(p);
                }
            }

            connection.Open();


            using SqlDataReader sqlDataReader = command.ExecuteReader(CommandBehavior.KeyInfo);

            List<Table> tables = new();

            while (sqlDataReader.HasRows == true)
            {
                Table table = new(sqlDataReader.GetSchemaTable().Rows[0].Field<string>("BaseTableName"));


                // Читаем строки
                while (sqlDataReader.Read() == true)
                {
                    TableRow tableRow = new();

                    for (int i = 0; i < sqlDataReader.FieldCount; i++)
                    {
                        string colName = sqlDataReader.GetColumnSchema()[i].ColumnName;
                        tableRow.columns.Add(colName, sqlDataReader.GetValue(colName));
                    }

                    table.TableRows.Add(tableRow);
                }

                tables.Add(table);

                // Переход на следующую таблицу
                sqlDataReader.NextResult();
            }

            return tables;
        }

    }
}
